/*
 * Data.java
 *
 * Created on May 7, 2004, 11:31 AM
 */
package usgs.thorn;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 *
 * @author  markstro
 */
public class ThornData {
    private Hashtable<String,Parameter> parameters;
    private Hashtable<String,Variable> variables;
    private File inputFile;
    private int[] year = null;
    private int[] month = null;
    private int numTimeSteps = -1;
    
    /**
     * Creates a new instance of ThornData
     */
    public ThornData() {
        inputFile = null;

// Create the parameters
        parameters = new Hashtable<String,Parameter>(3);
        parameters.put("soilMoistureStorage", new Parameter("soilMoistureStorage", 200.0, 0.0, 1500.0));
        parameters.put("latitude", new Parameter("latitude", 35.0, -90.0, 90.0));
        parameters.put("runoffFactor", new Parameter("runoffFactor", 50.0, 0.0, 100.0));
        parameters.put("rainTempThreshold", new Parameter("rainTempThreshold", 0.0, -5.0, 5.0));
        parameters.put("snowTempThreshold", new Parameter("snowTempThreshold", 0.0, -5.0, 5.0));
        parameters.put("directRunoffFactor", new Parameter("directRunoffFactor", 5.0, 0.0, 100.0));
        parameters.put("maxMeltRate", new Parameter("maxMeltRate", 50.0, 0.0, 100.0));
        
// Create the variables
        variables = new Hashtable<String,Variable>(10);
        variables.put("pet", new Variable ("pet", "Potential ET", "Millimeters"));
        variables.put("precip", new Variable ("precip", "Precipitation", "Millimeters"));
        variables.put("precipPet", new Variable ("precipPet", "Precip - Pot ET", "Millimeters"));
        variables.put("soilMoist", new Variable ("soilMoist", "Soil Moisture Storage", "Millimeters"));
        variables.put("aet", new Variable ("aet", "Actual ET", "Millimeters"));
        variables.put("petAet", new Variable ("petAet", "Potential ET - Actual ET", "Millimeters"));
        variables.put("snow", new Variable ("snow", "Snow Storage", "Millimeters"));
        variables.put("surplus", new Variable ("surplus", "Surplus", "Millimeters"));
        variables.put("runoff", new Variable ("runoff", "Runoff", "Millimeters"));
        variables.put("temp", new Variable ("temp", "Temperature", "Degrees C"));
        variables.put("rodirect", new Variable ("rodirect", "Direct Runoff", "Millimeters"));
        variables.put("snowMelt", new Variable ("snowMelt", "Snow Melt", "Millimeters"));
    }
    
    public double getParameterValue (String name) {return parameters.get(name).getValue();}
    public void setParameterValue (String name, double value) {parameters.get(name).setValue(value);}
    public double[] getVariableValues (String name) {return variables.get(name).getValues();}
    public void setVariableValues (String name, double[] values) {variables.get(name).setValues(values);}
    public void setVariableValue(String name, int index, double value) {variables.get(name).getValues()[index] = value;}
    public File getInputFile () {return inputFile;}
    public void setInputFile (File file) {this.inputFile = file;}
    public boolean isVariablePlotted (String name) {return variables.get(name).isPlotted();}
    public void setVariablePlotted (String name, boolean value) {variables.get(name).setPlotted(value);}
    
    public void resizeVariables(int length) {
        Enumeration en = variables.elements();
        while (en.hasMoreElements()) {
            ((Variable) en.nextElement()).setValues(new double[length]);
        }
    }
    
    public void readInput () {
        if (inputFile == null) return;
        
        try {
            BufferedReader in = new BufferedReader(new FileReader(inputFile));
            
/*
 ** Count lines in the input file.
 */
            String line;
            numTimeSteps = 0;
            while ((line = in.readLine()) != null) numTimeSteps++;
            in.close();
            
/*
 ** Size the input arrays
 */
            year = new int[numTimeSteps];
            month = new int[numTimeSteps];
            resizeVariables(numTimeSteps);
            
/*
 **  READ TEMPERATURE (C) AND PRECIPITATION (mm)
 */
            double[] temp = variables.get("temp").getValues();
            double[] precip = variables.get("precip").getValues();
            in = new BufferedReader(new FileReader(inputFile));
            
            int i = 0;
            while ((line = in.readLine()) != null) {
                String[] foo = line.trim().split("[ \t][ \t]*");
                year[i] = (int)(Double.valueOf(foo[0]).doubleValue());
                month[i] = (int)(Double.valueOf(foo[1]).doubleValue());
                temp[i] = Double.valueOf(foo[2]).doubleValue();
                precip[i] = Double.valueOf(foo[3]).doubleValue();
                i++;
            }
            in.close();

        } catch (Exception ioe) {

        }
    }

    /**
     * Getter for property year.
     * @return Value of property year.
     */
    public int[] getYear() {
        return this.year;
    }

    /**
     * Setter for property year.
     * @param year New value of property year.
     */
    public void setYear(int[] year) {
        this.year = year;
    }

    /**
     * Getter for property month.
     * @return Value of property month.
     */
    public int[] getMonth() {
        return this.month;
    }

    /**
     * Setter for property month.
     * @param month New value of property month.
     */
    public void setMonth(int[] month) {
        this.month = month;
    }

    /**
     * Getter for property numTimeSteps.
     * @return Value of property numTimeSteps.
     */
    public int getNumTimeSteps() {
        return this.numTimeSteps;
    }

    /**
     * Setter for property numTimeSteps.
     * @param numTimeSteps New value of property numTimeSteps.
     */
    public void setNumTimeSteps(int numTimeSteps) {
        this.numTimeSteps = numTimeSteps;
    }

    /**
     * Getter for property variables.
     * @return Value of property variables.
     */
    public Hashtable getVariables() {
        return this.variables;
    }

//    /**
//     * Setter for property variables.
//     * @param variables New value of property variables.
//     */
//    public void setVariables(Hashtable variables) {
//        this.variables = variables;
//    }
}
