//  This comes from TimeSeriesDemo11

package usgs.thorn;

import java.awt.Color;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Month;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.util.UnitType;

public class ThornPlotter extends JFrame {
    private ThornData data;
    private ThornGUI gui;

    public ThornPlotter(ThornGUI gui, ThornData data) {
        super("Thornthwaite Data Plot");
        this.data = data;
        this.gui = gui;
        setContentPane(createPanel());
    }
    
    /**
     *
     * @return A panel.
     */
    public JPanel createPanel() {
        int nVars = gui.getNumPlot();
        
        Month start = new Month(data.getMonth()[0], data.getYear()[0]);
        
        CombinedDomainXYPlot plot = new CombinedDomainXYPlot(new DateAxis("Time"));
        TimeSeriesCollection[] datasets = new TimeSeriesCollection[nVars];
        
        int plotNum = 0;
        Enumeration en = data.getVariables().elements();
        while (en.hasMoreElements()) {
            Variable var = (Variable) en.nextElement();

            if (var.isPlotted()) {
                datasets[plotNum] = createDataset(var, start);
                NumberAxis rangeAxis = new NumberAxis(var.getUnit());
                rangeAxis.setAutoRangeIncludesZero(false);
                XYPlot subplot = new XYPlot(datasets[plotNum], null, rangeAxis, new StandardXYItemRenderer());
                subplot.setBackgroundPaint(Color.lightGray);
                subplot.setDomainGridlinePaint(Color.white);
                subplot.setRangeGridlinePaint(Color.white);
                plot.add(subplot);
                plotNum++;
            }
        }
        
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd' 'HH:mm:ss");
        JFreeChart chart = new JFreeChart("Run Time: " + df.format(new Date(System.currentTimeMillis())), plot);
        LegendTitle legend = (LegendTitle) chart.getSubtitle(0);
        legend.setPosition(RectangleEdge.RIGHT);
        legend.setMargin(new RectangleInsets(UnitType.ABSOLUTE, 0, 4, 0, 4));
        
        chart.setBorderPaint(Color.black);
        chart.setBorderVisible(true);
        chart.setBackgroundPaint(Color.white);
        
        plot.setBackgroundPaint(Color.lightGray);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        plot.setAxisOffset(new RectangleInsets(4, 4, 4, 4));
        ValueAxis axis = plot.getDomainAxis();
        
        ChartPanel chartPanel = new ChartPanel(chart);
        add(chartPanel);
        
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 470));
        chartPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        return chartPanel;
    }
    /**
     * Creates a sample dataset.
     *
     * @param name  the dataset name.
     * @param base  the starting value.
     * @param start  the starting period.
     * @param count  the number of values to generate.
     *
     * @return The dataset.
     */
    private static TimeSeriesCollection createDataset(Variable var, Month start) {
        
//        TimeSeries series = new TimeSeries(var.getName(), start.getClass());
        TimeSeries series = new TimeSeries(var.getDesc(), start.getClass());
        RegularTimePeriod period = start;
        double[] values = var.getValues();
        
        for (int i = 0; i < values.length; i++) {
            series.add(period, values[i]);
            period = period.next();
        }
        
        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(series);
        
        return dataset;
        
    }
}
