package usgs.thorn;

public class Thornthwaite {
   final static double[] day = {15.0, 45.0, 74.0, 105.0, 135.0, 166.0, 196.0,
                                227.0, 258.0, 288.0, 319.0, 349.0};

   final static double[] days = {31.0, 28.0, 31.0, 30.0, 31.0, 30.0, 31.0,
                                31.0, 30.0, 31.0, 30.0, 31.0};

//   private double[][] data = new double[9][];

   public Thornthwaite(ThornData data) {

       double dayl, decd, decr, cz, alat, sintheta, csh, h, dl, pt, pe, pmpe;
       double theta;
       double ae, delstor, tcprior;

       double stor = 0.0;
       double prestor = 150.0;
       double remain = 25.4;
       double snow = 0.0;
       double xns = 0.0;
       double snstor = 0.0;
       double sntot = 0.0;
       double ro = 0.0;
       double sur = 0.0;
       double snowMelt = 0.0;
       double totalro = 0.0;
       double totalpmpe = 0.0;
       double psnow = 0.0;
       double prain = 0.0;
       double rodirect = 0.0;
       double deficit = 0.0;


//  Parameters set by GUI
       double waterHoldingCapacity = data.getParameterValue("soilMoistureStorage");
       double runoffFactor = data.getParameterValue("runoffFactor") / 100.0;
       double train = data.getParameterValue("rainTempThreshold");
       double tsnow = data.getParameterValue("snowTempThreshold");
       double directfac = data.getParameterValue("directRunoffFactor") / 100.0;
       double xmeltmax = data.getParameterValue("maxMeltRate") / 100.0;

       int[] months = data.getMonth();
       double[] temps = data.getVariableValues("temp");
       double[] precips = data.getVariableValues("precip");

       for (int i = 0; i < data.getNumTimeSteps(); i++) {
           int month = months[i];
           double tc = temps[i];
           double p = precips[i];

/*
** COMPUTE SNOW AND RAIN
*/
         if (tc <= tsnow) psnow = p;
         if (tc > tsnow && tc < train) psnow = p *((train - tc) / (train - tsnow));
         if (tc >= train) psnow = 0.0;
         prain = p - psnow;

         rodirect = prain * directfac;
         prain = prain - rodirect;
         snstor = snstor + psnow;

/*
 **  CALCULATE DAYLENGTH
 */
//           dayl = day[month - 1] - 80.;
//           if (dayl < 0.0) dayl = 285. + day[month - 1];
//
//           decd = 23.45 * java.lang.Math.sin(dayl / 365. * 6.2832);
//           decr = decd * 0.017453;
//           cz = -0.02908;
//           alat = data.getParameterValue("latitude") * 0.017453;
//           xx = java.lang.Math.cos(decr) * java.lang.Math.cos(alat);
//           csh = (cz - java.lang.Math.sin(decr) * java.lang.Math.sin(alat)) / xx;
//           h = 1.570796 - java.lang.Math.atan(csh / java.lang.Math.sqrt(1. - csh * csh));
//           dl = 24.0 * h / 3.1416;


         dayl = -1.0;
         try { // If the data file has a blank line, the month will be set to 0
             dayl = day[month - 1] - 80.;
         } catch (Exception e) {
             break;
         }

         if (dayl < 0.0) dayl = 285. + day[month - 1];

         decd = 23.45 * java.lang.Math.sin(dayl / 365. * 6.2832);
         decr = decd * 0.017453;
         alat = data.getParameterValue("latitude") * 0.017453;

         sintheta = java.lang.Math.tan(alat) * java.lang.Math.tan(decr);
         theta = java.lang.Math.asin(sintheta);

         if (sintheta <= -1.0) {
             dl = 0.0;
         } else if (sintheta >= 1.0) {
             dl= 24.0;
         } else {
             dl = 12. + 2. * theta * 24. / (2.0 * 3.174);
         }


/*
 **  CALCULATE HAMON PE
 */
           pt = 4.95 * java.lang.Math.exp(0.062 * tc) / 100.;
           pe = 0.55 * ((dl / 12.0) * (dl / 12.0)) * pt * days[month - 1];
           if (pe <= 0.0) pe = 0.0;
           pe = pe * 25.4;

/*
** COMPUTE SNOW MELT
*/
           snowMelt = 0.0;

           if (snstor > 0.0 && tc > tsnow) {
               if (snstor <= 10.0) {
                   snowMelt = snstor;
               } else {
                   double fracMelt = ((tc - tsnow) / ((train-tsnow) + 0.0001)) * xmeltmax;
                   if (fracMelt > xmeltmax) fracMelt = xmeltmax;
                   snowMelt = fracMelt * snstor;
               }

               snstor = snstor - snowMelt;
               if(snstor < 0.0) snstor = 0.0;
           }


/*
** ADD SNOW MELT TO PRAIN
*/
        prain = prain + snowMelt;

/*
 * Compute pmpe
 */
           pmpe = prain - pe;
/*
 * COMPUTE SOIL-MOISTURE STORAGE, AE, SURPLUS, AND DEFICIT
 */
           if (pmpe < 0.0) {
               stor = prestor - Math.abs(pmpe * prestor / waterHoldingCapacity);
               if (stor < 0.0) stor = 0.0;
               delstor = stor - prestor;
               ae = prain + delstor * (-1.0);
               prestor = stor;
               sur = 0.0;

           } else {
               ae = pe;

               stor = prestor + pmpe;
               if (stor > waterHoldingCapacity) {
                   sur = stor - waterHoldingCapacity;
                   stor = waterHoldingCapacity;
               }
               prestor = stor;
           }

           deficit = pe - ae;

/*
 **  RUNOFF CALCULATIONS
 */
           ro = (sur + remain) * runoffFactor;
           remain = (sur + remain) - ro;
           if (remain < 0.0) remain = 0.0;
           ro = ro + rodirect;


//  Put the calculated variables into the data object for plotting and report
           data.setVariableValue("pet", i, pe);
           data.setVariableValue("precip", i, p);
           data.setVariableValue("precipPet", i, pmpe);
           data.setVariableValue("soilMoist", i, stor);
           data.setVariableValue("aet", i, ae);
           data.setVariableValue("petAet", i, deficit);
           data.setVariableValue("snow", i, snstor);
           data.setVariableValue("surplus", i, sur);
           data.setVariableValue("runoff", i, ro);
           data.setVariableValue("rodirect", i, rodirect);
           data.setVariableValue("snowMelt", i, snowMelt);
       }
   }
}
